package com.lantone.qc.kernel.catalogue.behospitalized;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.kernel.util.CatalogueUtil;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.entity.Clinical;
import com.lantone.qc.pub.model.entity.PD;
import com.lantone.qc.pub.model.label.ChiefLabel;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * @ClassName : BEH0007
 * @Description :主诉症状时间描述模糊
 * 主诉中不能出现数天，数月，数年等带“数”字这种描述，如果出现，均视为模糊不清
 * @Author : 楼辉荣
 * @Date: 2020-03-06 17:28
 */
@Component
public class BEH0007 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {

        ChiefLabel chiefLabel = inputInfo.getBeHospitalizedDoc().getChiefLabel();
        List<Clinical> clinicals = chiefLabel.getClinicals();
        Set<String> clinicalSet = new HashSet<>();
        if(clinicals.size()>0){
            for (Clinical clinical:clinicals) {
                String clinicalName = clinical.getName();
                List<PD> clinicalTimestamp = clinical.getTimestamp();
                if(clinicalTimestamp.size()>0){
                    for (PD pd:clinicalTimestamp) {
                        if(StringUtils.isEmpty(pd.getUnit())){
                            clinicalSet.add(clinicalName);
                        }
                    }
                }
            }
        }
        if(clinicalSet.size()>0){
            status = "-1";
            info = clinicalSet.toString().replace("[","").replace("]","");
        }else {
            status = "0";
        }

    }
}