package com.lantone.qc.kernel.structure.ai.process;

import com.alibaba.fastjson.JSONObject;
import com.lantone.qc.kernel.structure.ai.model.CrfOut;
import com.lantone.qc.kernel.structure.ai.model.EntityEnum;
import com.lantone.qc.pub.model.entity.*;
import com.lantone.qc.pub.model.label.PastLabel;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class EntityProcessPast extends EntityProcess {
    public static String allergy_history = "过敏史";
    public static String disease_history = "疾病史";
    public static String surgery_history = "手术史";
    public static String injury_history = "外伤史";
    public static String blood_history = "输血史";
    public static String vaccination_history = "预防接种史";
    public static String infect_history = "传染病史";

    public PastLabel extractEntity(JSONObject aiOut) {
        PastLabel pastLabel = new PastLabel();

        try {
            List<Diag> diags = addEntity(aiOut, EntityEnum.DIEASE, Diag.class);
            //手术史
            List<Operation> operations = addEntity(aiOut, EntityEnum.OPERATION, Operation.class);
            //外伤史
            List<Wound> wounds = addEntity(aiOut, EntityEnum.INJURY, Wound.class);
            //过敏史
            List<Allergy> allergies = addEntity(aiOut, EntityEnum.ALLERGY, Allergy.class);
            List<AllergyFood> allergyFoods = addEntity(aiOut, EntityEnum.FOOD_ALLERGY, AllergyFood.class);
            List<AllergyMedicine> allergyMedicines = addEntity(aiOut, EntityEnum.DRUG_ALLERGY, AllergyMedicine.class);
            List<AllergyDesc> allergyDescs = addEntity(aiOut, EntityEnum.ALLERGY_SYMPTOM, AllergyDesc.class);

            //输血史
            List<BloodTransfusion> bloodTransfusions = addEntity(aiOut, EntityEnum.BLOOD_TRANSFUSION, BloodTransfusion.class);
             //预防接种史
            List<Vaccinate> vaccinates = addEntity(aiOut, EntityEnum.VACCINATION, Vaccinate.class);
            //传染病史
            List<DiagInfectious> diagInfectious = addEntity(aiOut, EntityEnum.INFECTIOUS_KEYWORD, DiagInfectious.class);
            pastLabel.setDiags(diags);
            pastLabel.setAllergies(allergies);
            pastLabel.setAllergyFoods(allergyFoods);
            pastLabel.setAllergyMedicines(allergyMedicines);
            pastLabel.setAllergyDescs(allergyDescs);
            pastLabel.setBloodTransfusions(bloodTransfusions);
            pastLabel.setDiagInfectiouses(diagInfectious);
            pastLabel.setOperations(operations);
            pastLabel.setVaccinates(vaccinates);
            pastLabel.setWounds(wounds);
        }catch (Exception e){
            e.printStackTrace();
        }
        return pastLabel;



    }
    private void addPast(JSONObject outputs, List<Past> pasts) {
        Past past = null;
        List<Map<String, String>> pastEntityList = processJson(outputs, EntityEnum.DIEASE.toString());
        for (Map<String, String> pastEntityMap : pastEntityList) {
            if (StringUtils.isEmpty(pastEntityMap.get(EntityEnum.DIEASE.toString()))) {
                continue;
            }
            past = new Past();
            String cureValue = "";
            for (String key : pastEntityMap.keySet()) {
                switch (EntityEnum.parseOfValue(key)) {
                    case DIEASE:
                        past.setType(6);
                        past.setTitle(EntityProcessPast.disease_history);
                        past.setValue(pastEntityMap.get(key));
                        break;
                    case CURE:
                        past.setTreat(pastEntityMap.get(key));
                        break;
                    case CURE_AIM:
                        cureValue += "," + pastEntityMap.get(key);
                        break;
                    case TREND:
                        cureValue += "," + pastEntityMap.get(key);
                        break;
                    case TIME:
                        cureValue += "," + pastEntityMap.get(key);
                        break;
                    case FREQUENCY:
                        cureValue += "," + pastEntityMap.get(key);
                        break;
                    case MODIFICATION:
                        cureValue += "," + pastEntityMap.get(key);
                        break;
                    case NEGATIVE:
                        Negative negative = new Negative();
                        negative.setName(StringUtils.isEmpty(pastEntityMap.get(key)) ? "" : pastEntityMap.get(key));
                        past.setNegative(negative);
                        break;
                    //TODO   PD
                }
            }
            past.setDescribe(cureValue.startsWith(",") ? cureValue.substring(1, cureValue.length()) : cureValue);
            pasts.add(past);
        }
    }
}
