package com.lantone.qc.kernel.util;

import com.lantone.qc.pub.jdbc.MysqlJdbc;
import com.lantone.qc.pub.util.PropertiesUtil;
import com.lantone.qc.security.util.CryptUtil;
import com.lantone.qc.security.util.EncrypDES;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;

/**
 * Created by louhr on 2019/8/16.
 */
public class CacheFileManager {
    Logger logger = LoggerFactory.getLogger(CacheFileManager.class);

    private String user;
    private String password;
    private String url;

    private String path = "";

    public static void main(String[] args) {
        CacheFileManager cacheFileManager = new CacheFileManager();
        String p = cacheFileManager.getClass().getResource("/").getPath();
        p = p.substring(0, p.indexOf("/target/")) + "/src/main/resources/cache/";
        cacheFileManager.createCacheFile(p);
    }

    public CacheFileManager() {
        try {
            PropertiesUtil propertiesUtil = new PropertiesUtil("kernel.properties");
//            EncrypDES encrypDES = new EncrypDES();
//            user = encrypDES.decryptor(propertiesUtil.getProperty("mysql.redis.user"));
//            password = encrypDES.decryptor(propertiesUtil.getProperty("mysql.redis.password"));
//            url = encrypDES.decryptor(propertiesUtil.getProperty("mysql.redis.url"));
            user = propertiesUtil.getProperty("mysql.redis.user");
            password = propertiesUtil.getProperty("mysql.redis.password");
            url = propertiesUtil.getProperty("mysql.redis.url");
        } catch (Exception e) {
            logger.info("数据库连接信息出错...." + e.getMessage());
        }
    }

    public void createCacheFile(String path) {
        MysqlJdbc nlpJdbc = new MysqlJdbc(user, password, url);
        Connection conn = nlpJdbc.connect();
        Statement st = null;
        ResultSet rs = null;
        String r1, r2, r3, r4, r5;
        try {
            EncrypDES encrypDES = new EncrypDES();
            st = conn.createStatement();
            //疾病相關特征
            String sql = "select kc.id, group_concat(feature) feature_name from kl_concept kc, doc_relevant_feature drf\n" +
                    "where kc.id = drf.diagnose group by kc.id";
            Map<String, String> map = new HashMap<>();
            rs = st.executeQuery(sql);
            while (rs.next()) {
                r1 = rs.getString(1);
                r2 = rs.getString(2);
                r2 = r2 == null ? "" : r2;
                map.put(r1, r2);
            }

            //所有词典库 不能用concat_group 大小写不区分
            sql = "select l_1.name l_1_name, l_1.type_id type_id, l_2.name l_2_name, \n" +
                    "kc.lib_name, dis.is_chronic, dis.is_infections, group_concat(l_1.name) group_name, l_1.concept_id \n" +
                    "from kl_library_info l_1\n" +
                    "left join kl_library_info l_2 on l_1.concept_id = l_2.concept_id and l_2.is_concept = 1\n" +
                    "left join kl_concept kc on l_1.concept_id = kc.id\n" +
                    "left join kl_disease dis on l_1.concept_id = dis.concept_id\n" +
                    "where kc.is_deleted = 'N' and l_1.type_id = 18 group by l_1.concept_id";

            FileWriter fw = new FileWriter(path + "concept_diag_properties.dict");
            rs = st.executeQuery(sql);
            while (rs.next()) {
                r1 = rs.getString(4);
                r2 = rs.getString(5);
                r3 = rs.getString(6);
                r4 = rs.getString(7);
                r5 = rs.getString(8);

                r1 = CryptUtil.decrypt_char(r1==null?"":r1);
                r2 = r2==null?"0":r2;
                r3 = r3==null?"0":r3;
                r4 = CryptUtil.decrypt_char(r4==null?"":r4);
                r5 = map.get(r5) == null ? "" : map.get(r5);

                fw.write(encrypDES.encrytor(r1 + "|" + r2 + "|" + r3 + "|" + r4 + "|" + r5));
                fw.write("\n");
            }
            fw.close();

            //性质词典
            sql = "select name from kl_library_info where type_id = 4";
            fw = new FileWriter(path + "concept_prop.dict");
            rs = st.executeQuery(sql);
            while (rs.next()) {
                r1 = rs.getString(1);
                r1 = CryptUtil.decrypt_char(r1==null?"":r1);
                fw.write(encrypDES.encrytor(r1));
                fw.write("\n");
            }
            fw.close();

            //程度词典
            sql = "select name from kl_library_info where type_id = 6";
            fw = new FileWriter(path + "concept_degree.dict");
            rs = st.executeQuery(sql);
            while (rs.next()) {
                r1 = rs.getString(1);
                r1 = CryptUtil.decrypt_char(r1==null?"":r1);
                fw.write(encrypDES.encrytor(r1));
                fw.write("\n");
            }
            fw.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            nlpJdbc.close(rs, st, conn);
        }
    }

    private List<Map.Entry<String, String>> rsToMap(ResultSet rs, boolean isJoin) throws SQLException {
        String r1 = "";
        String r2 = "";
        Map<String, String> libraryMap = new HashMap<>(10);
        while (rs.next()) {
//            r1 = CryptUtil.decrypt_char(rs.getString(1));
//            r2 = CryptUtil.decrypt_char(rs.getString(2));
            if (libraryMap.get(r1) == null) {
                libraryMap.put(r1, r2);
            } else if (isJoin && libraryMap.get(r1) != null) {
                libraryMap.put(r1, libraryMap.get(r1) + "," + r2);
            }
        }

        List<Map.Entry<String, String>> libraryList = new ArrayList<Map.Entry<String, String>>(libraryMap.entrySet());
        Collections.sort(libraryList, new Comparator<Map.Entry<String, String>>() {
            public int compare(Map.Entry<String, String> o1, Map.Entry<String, String> o2) {
                return o1.getKey().compareTo(o2.getKey());
            }
        });

        return libraryList;
    }
}
